@@ -2,6 +2,7 @@ import hmac
 import hashlib
 import time
 import base64
+import threading
 from requests.auth import AuthBase
 
 
@@ -9,15 +10,27 @@ class AuthProvider(AuthBase):
     def __init__(self, api_key, secret_key):
         self.api_key = api_key
         self.secret_key = secret_key
+        self.nonce_lock = threading.Lock()
+        self.last_nonce = int(time.time() *  1000)
 
 
     def __call__(self, request):
-        timestamp = int(time.time() *  1000)
-        message = ''.join([request.path_url, request.method, str(timestamp), (request.body or '')])
-        headers = get_auth_headers(timestamp, message, self.api_key, self.secret_key)
+        nonce = self._get_nonce()
+        message = ''.join([request.path_url, request.method, str(nonce), (request.body or '')])
+        headers = get_auth_headers(nonce, message, self.api_key, self.secret_key)
         request.headers.update(headers)
 
         return request
+
+    def _get_nonce(self):
+        new_nonce = int(time.time() *  1000)
+        with self.nonce_lock:
+            if (new_nonce <= self.last_nonce):
+                new_nonce = new_nonce + 1
+
+            self.last_nonce = new_nonce
+
+        return new_nonce
     
 
 def get_auth_headers(timestamp, message, api_key, secret_key):
